@@ -6,10 +6,15 @@
 
 #include "types.h"
 
+#include <xkbcommon/xkbcommon.h>
+
 // Background modes
 #define COLOR_TRANSPARENT 0xff000000
 #define BACKGROUND_GRID   0xfe000000
 
+// Max number of key bindings
+#define MAX_KEYBINDINGS 128
+
 /** Order of file list. */
 enum config_order {
     cfgord_none,  ///< Unsorted (system depended)
@@ -24,12 +29,54 @@ enum config_scale {
     cfgsc_real     ///< Real image size (100%)
 };
 
+/** Action associated with a key. */
+enum config_action {
+    cfgact_none,
+    cfgact_first_file,
+    cfgact_last_file,
+    cfgact_prev_dir,
+    cfgact_next_dir,
+    cfgact_prev_file,
+    cfgact_next_file,
+    cfgact_prev_frame,
+    cfgact_next_frame,
+    cfgact_animation,
+    cfgact_slideshow,
+    cfgact_fullscreen,
+    cfgact_step_left,
+    cfgact_step_right,
+    cfgact_step_up,
+    cfgact_step_down,
+    cfgact_zoom_in,
+    cfgact_zoom_out,
+    cfgact_zoom_optimal,
+    cfgact_zoom_fit,
+    cfgact_zoom_real,
+    cfgact_zoom_reset,
+    cfgact_rotate_left,
+    cfgact_rotate_right,
+    cfgact_flip_vertical,
+    cfgact_flip_horizontal,
+    cfgact_reload,
+    cfgact_antialiasing,
+    cfgact_info,
+    cfgact_exec,
+    cfgact_quit,
+};
+
+/** Key bindings. */
+struct config_keybind {
+    xkb_keysym_t key;
+    enum config_action action;
+};
+
 /** App configuration. */
 struct config {
     const char* app_id;      ///< Window class/app_id name
     bool sway_wm;            ///< Enable/disable integration with Sway WM
     struct rect geometry;    ///< Window geometry
     bool fullscreen;         ///< Full screen mode
+    bool antialiasing;       ///< Anti-aliasing
     argb_t background;       ///< Image background mode/color
     argb_t window;           ///< Window background mode/color
     enum config_scale scale; ///< Initial scale
@@ -43,7 +90,8 @@ struct config {
     bool loop;               ///< File list loop mode
     bool recursive;          ///< Read directories recursively
     bool all_files;          ///< Open all files from the same directory
-    bool mark_mode;          ///< Enable/disable marking mode
+    const char* exec_cmd;    ///< Command to execute
+    struct config_keybind keybind[MAX_KEYBINDINGS]; ///< Key bindings table
 };
 
 /**
@@ -68,3 +116,4 @@ bool config_set_font_size(struct config* ctx, const char* val);
 bool config_set_order(struct config* ctx, const char* val);
 bool config_set_slideshow_sec(struct config* ctx, const char* val);
 bool config_set_appid(struct config* ctx, const char* val);
+bool config_set_exec_cmd(struct config* ctx, const char* val);
