@@ -13,7 +13,7 @@
     enum loader_status LOADER_FUNCTION(name)(struct image * ctx, \
                                              const uint8_t* data, size_t size)
 
-const char* supported_formats = "bmp"
+const char* supported_formats = "bmp, pnm"
 #ifdef HAVE_LIBJPEG
                                 ", jpeg"
 #endif
@@ -35,6 +35,9 @@ const char* supported_formats = "bmp"
 #ifdef HAVE_LIBJXL
                                 ", jxl"
 #endif
+#ifdef HAVE_LIBEXR
+                                ", exr"
+#endif
 #ifdef HAVE_LIBTIFF
                                 ", tiff"
 #endif
@@ -42,6 +45,10 @@ const char* supported_formats = "bmp"
 
 // declaration of loaders
 LOADER_DECLARE(bmp);
+LOADER_DECLARE(pnm);
+#ifdef HAVE_LIBEXR
+LOADER_DECLARE(exr);
+#endif
 #ifdef HAVE_LIBGIF
 LOADER_DECLARE(gif);
 #endif
@@ -79,6 +86,7 @@ static const image_decoder decoders[] = {
     &LOADER_FUNCTION(gif),
 #endif
     &LOADER_FUNCTION(bmp),
+    &LOADER_FUNCTION(pnm),
 #ifdef HAVE_LIBWEBP
     &LOADER_FUNCTION(webp),
 #endif
@@ -91,6 +99,9 @@ static const image_decoder decoders[] = {
 #ifdef HAVE_LIBJXL
     &LOADER_FUNCTION(jxl),
 #endif
+#ifdef HAVE_LIBEXR
+    &LOADER_FUNCTION(exr),
+#endif
 #ifdef HAVE_LIBTIFF
     &LOADER_FUNCTION(tiff),
 #endif
