@@ -24,7 +24,6 @@
 /** Single list entry. */
 struct entry {
     size_t index; ///< Entry index in the list
-    bool marked;  ///< Mark state
     char path[1]; ///< Path to the file, any size array
 };
 
@@ -82,7 +81,6 @@ static void add_file(struct image_list* ctx, const char* file)
     if (entry) {
         memcpy(entry->path, file, path_len + 1);
         entry->index = ctx->index;
-        entry->marked = false;
         ctx->entries[ctx->index] = entry;
         ++ctx->index;
         ++ctx->size;
@@ -211,7 +209,7 @@ static size_t peek_next_dir(const struct image_list* ctx, const char* file,
         size_t next_len;
 
         index = peek_next_file(ctx, index, forward);
-        if (index == SIZE_MAX) {
+        if (index == SIZE_MAX || index == start) {
             break; // not found
         }
 
@@ -320,7 +318,7 @@ static void* preloader_thread(void* data)
 }
 
 /**
- * Stop or start background thread to preload next image.
+ * Stop or restart background thread to preload adjacent images.
  * @param ctx image list context
  * @param restart action: true=restart, false=stop preloader
  */
@@ -459,12 +457,70 @@ size_t image_list_size(const struct image_list* ctx)
 
 struct image_entry image_list_current(const struct image_list* ctx)
 {
-    struct image_entry entry = { .index = ctx->index,
-                                 .marked = ctx->entries[ctx->index]->marked,
-                                 .image = ctx->current };
+    struct image_entry entry = { .index = ctx->index, .image = ctx->current };
     return entry;
 }
 
+int image_list_cur_exec(const struct image_list* ctx)
+{
+    const char* template = ctx->config->exec_cmd;
+    const char* path = ctx->current->file_path;
+    size_t pos = 0;
+    size_t len = 0;
+    char* cmd = NULL;
+    int rc = -1;
+
+    // construct command text
+    while (*template) {
+        const char* append = template;
+        size_t append_sz = 1;
+        if (*template == '%') {
+            if (*(template + 1) == '%') {
+                // escaped %
+                ++template;
+            } else {
+                // replace % with path
+                append = path;
+                append_sz = strlen(path);
+            }
+        }
+        ++template;
+        if (pos + append_sz >= len) {
+            char* ptr;
+            len = pos + append_sz + 32;
+            ptr = realloc(cmd, len);
+            if (!ptr) {
+                fprintf(stderr, "Not enough memory\n");
+                free(cmd);
+                cmd = NULL;
+                break;
+            }
+            cmd = ptr;
+        }
+        memcpy(cmd + pos, append, append_sz);
+        pos += append_sz;
+    }
+
+    // execute command
+    if (cmd) {
+        cmd[pos] = 0;
+        rc = system(cmd);
+        free(cmd);
+    }
+
+    return rc;
+}
+
+bool image_list_cur_reload(struct image_list* ctx)
+{
+    struct image* image = image_from_file(ctx->current->file_path);
+    if (image) {
+        image_free(ctx->current);
+        ctx->current = image;
+    }
+    return !!image;
+}
+
 bool image_list_jump(struct image_list* ctx, enum list_jump jump)
 {
     struct image* image = NULL;
@@ -524,38 +580,3 @@ bool image_list_jump(struct image_list* ctx, enum list_jump jump)
 
     return true;
 }
-
-void image_list_mark_invcur(struct image_list* ctx)
-{
-    ctx->entries[ctx->index]->marked = !ctx->entries[ctx->index]->marked;
-}
-
-void image_list_mark_invall(struct image_list* ctx)
-{
-    for (size_t i = 0; i < ctx->size; ++i) {
-        struct entry* entry = ctx->entries[i];
-        if (entry) {
-            entry->marked = !entry->marked;
-        }
-    }
-}
-
-void image_list_mark_setall(struct image_list* ctx, bool mark)
-{
-    for (size_t i = 0; i < ctx->size; ++i) {
-        struct entry* entry = ctx->entries[i];
-        if (entry) {
-            entry->marked = mark;
-        }
-    }
-}
-
-void image_list_mark_print(const struct image_list* ctx)
-{
-    for (size_t i = 0; i < ctx->size; ++i) {
-        struct entry* entry = ctx->entries[i];
-        if (entry && entry->marked) {
-            puts(entry->path);
-        }
-    }
-}
