@@ -26,7 +26,6 @@ static const struct cmdarg arguments[] = {
     { 'o', "order",      "ORDER",   "set sort order for image list: none/[alpha]/random" },
     { 'r', "recursive",  NULL,      "read directories recursively" },
     { 'a', "all",        NULL,      "open all files from the same directory" },
-    { 'm', "mark",       NULL,      "enable marking mode" },
     { 'l', "slideshow",  NULL,      "activate slideshow mode on startup" },
     { 'f', "fullscreen", NULL,      "show image in full screen mode" },
     { 's', "scale",      "SCALE",   "set initial image scale: [optimal]/fit/real" },
@@ -34,6 +33,7 @@ static const struct cmdarg arguments[] = {
     { 'w', "window",     "XXXXXX",  "set window background color: [none]/RGB" },
     { 'g', "geometry",   "X,Y,W,H", "set window geometry" },
     { 'i', "info",       NULL,      "show image meta information (name, EXIF, etc)" },
+    { 'e', "exec",       "CMD",     "set execution command" },
     { 'c', "class",      "NAME",    "set window class/app_id" },
     { 'n', "no-sway",    NULL,      "disable integration with Sway WM" },
     { 'v', "version",    NULL,      "print version info and exit" },
@@ -110,9 +110,6 @@ static int parse_cmdargs(int argc, char* argv[], struct config* cfg)
             case 'a':
                 cfg->all_files = true;
                 break;
-            case 'm':
-                cfg->mark_mode = true;
-                break;
             case 'l':
                 cfg->slideshow = true;
                 break;
@@ -142,6 +139,11 @@ static int parse_cmdargs(int argc, char* argv[], struct config* cfg)
             case 'i':
                 cfg->show_info = true;
                 break;
+            case 'e':
+                if (!config_set_exec_cmd(cfg, optarg)) {
+                    return -1;
+                }
+                break;
             case 'c':
                 if (!config_set_appid(cfg, optarg)) {
                     return -1;
@@ -267,10 +269,6 @@ int main(int argc, char* argv[])
     // run main loop
     rc = ui_run(ui);
 
-    if (rc && cfg->mark_mode) {
-        image_list_mark_print(list);
-    }
-
 done:
     viewer_free(viewer);
     ui_free(ui);
